/*  
 *
 *  (c) 1999 Norm Bell
 *  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  cdgdef.h : Defines the definitions and structures to build the cdgpacker DLL.
 * 
 *  permission is granted for free use this demo software to develop 
 *  applications as long as the applications are given away
 *  for free and are not sold.
 */



/* include files */
#include "stdio.h"
#include "memory.h"
#include "limits.h"

#ifndef CDGDEF_H
#define CDGDEF_H

/******************************************************************************
 *
 * Memory Preset (subCode.instruction==1)
 *
 *    Set the screen to a particular color.
 *
 *-----------------------------------------------------------------------------
 *
 * In this instruction, the 16 byte data field is interepreted as follows.
 *
 * Color refers to a color to clear the screen to.  The entire screen should be
 * cleared to this color.
 *
 * When these commands appear in bunches (to insure that the screen gets cleared),
 * the repeat count is used to number them.  If this is true, and you have a
 * reliable data stream, you can ignore the command if repeat != 0.
 *
 *****************************************************************************/

#define  MEMORY_PRESET_SUBCODE            1
#define  MEMORY_PRESET_FILLER_SZ          14
#define  MEMORY_PRESET_COLOR_MASK         0x0F
#define  MEMORY_PRESET_REPEAT_MASK        0x0F

typedef struct {
   char  color;                              // Only lower 4 bits are used, mask with 0x0F
   char  repeat;                             // Only lower 4 bits are used, mask with 0x0F
   char  filler[MEMORY_PRESET_FILLER_SZ];
} CDG_MemPreset;

#define GetMPColor(a)        ((a).memPreset.color  & MEMORY_PRESET_COLOR_MASK)
#define GetMPRepeat(a)       ((a).memPreset.repeat & MEMORY_PRESET_REPEAT_MASK)

/******************************************************************************
 *
 * Border Preset (subCode.instruction==2)
 *
 *    Set the border of the screen to a particular color.
 *
 *-----------------------------------------------------------------------------
 *
 * In this instruction, the 16 byte data field is interepreted as follows.
 *
 * Color refers to a color to clear the screen to.  The border area of the
 * screen should be cleared to this color.  The border area is the area
 * contained with a rectangle defined by (0,0,300,216) minus the interior
 * pixels which are contained within a rectangle defined by (6,12,294,204).
 *
 *****************************************************************************/

#define  BORDER_PRESET_SUBCODE            2
#define  BORDER_PRESET_FILLER_SZ          15
#define  BORDER_PRESET_COLOR_MASK         0x0F

typedef struct {
   char  color;                              // Only lower 4 bits are used, mask with 0x0F
   char  filler[BORDER_PRESET_FILLER_SZ];
} CDG_BorderPreset;

#define GetBPColor(a)        ((a).borderPreset.color & BORDER_PRESET_COLOR_MASK)

/******************************************************************************
 *
 * Tile Block Normal (subCode.instruction==6)
 *
 *    Load a 12 x 6, 2 color tile and display it direct copy
 *
 * Tile Block XOR (subCode.instruction==38)
 *
 *    Load a 12 x 6, 2 color tile and display it using the XOR method.
 *
 *-----------------------------------------------------------------------------
 *
 * These commands load a 12 x 6 tile of pixels from the subCode.data area.
 * I recall that in the original CD+G spec, the tile is refered to as a "font",
 * but I think the word tile is more appropriate, because the tile can (and
 * does) contain any graphical image, not just text. Larger images are built
 * by using multiple tiles. The XOR variant is a special case of the same
 * command, the difference is described below.
 *
 * The tile is stored using 1-bit graphics.  The structure contains two colors
 * which are to be used when rendering the tile.  The tile is extracted from
 * 16 bytes of subCode.data[] in the following manner:
 *
 * color0, color1 describe the two colors (from the color table) which are to
 * be used when rendering the tile.  Color0 is used for 0 bits, Color1 is used
 * for 1 bits.
 *
 * Row and Column describe the position of the tile in tile coordinate space.
 * To convert to pixels, multiply row by 12, and column by 6.
 *
 * tilePixels[] contains the actual bit values for the tile, six pixels per
 * byte. The uppermost valid bit of each byte (0x20) contains the left-most
 * pixel of each scanline of the tile.
 *
 * In the normal instruction, the corresponding colors from the color table
 * are simply copied to the screen.
 *
 * In the XOR variant, the color values are combined with the color values
 * that are already onscreen using the XOR operator.  Since CD+G only allows
 * a maximum of 16 colors, we are XORing the pixel values (0-15) themselves,
 * which correspond to indexes into a color lookup table.  We are not XORing
 * the actual R,G,B values.
 *
 *****************************************************************************/

#define  TILE_BLOCK_NORMAL_SUBCODE     6
#define  TILE_BLOCK_XOR_SUBCODE        38
#define  TILE_BLOCK_FILLER_SZ          12

#define  TILE_COLOR0_MASK              0x0F
#define  TILE_COLOR1_MASK              0x0F
#define  TILE_ROW_MASK                 0x1F
#define  TILE_COLUMN_MASK              0x3F
#define  TILE_PIXELS_MASK              0x3F

typedef struct {
   char  color0;                             // Only lower 4 bits are used, mask with 0x0F
   char  color1;                             // Only lower 4 bits are used, mask with 0x0F
   char  row;                                // Only lower 5 bits are used, mask with 0x1F
   char  column;                             // Only lower 6 bits are used, mask with 0x3F
   char  pixels[TILE_BLOCK_FILLER_SZ];       // Only lower 6 bits of each byte are used
} CDG_Tile;

typedef enum {
   TILE_BLOCK_NORMAL,
   TILE_BLOCK_XOR
} TileBlock_Actions;

#define GetTBColor0(a)        ((a).tile.color0      & TILE_COLOR0_MASK)
#define GetTBColor1(a)        ((a).tile.color1      & TILE_COLOR1_MASK)
#define GetTBRow(a)           ((a).tile.row         & TILE_ROW_MASK)
#define GetTBColumn(a)        ((a).tile.column      & TILE_COLUMN_MASK)
#define GetTBPixel(a, b)      ((a).tile.pixels[(b)] & TILE_PIXELS_MASK)

/******************************************************************************
 *
 * Scroll Preset (subCode.instruction==20)
 *
 *    Scroll the image, filling in the new area with a color.
 *
 * Scroll Copy (subCode.instruction==24)
 *
 *    Scroll the image, rotating the bits back around.
 *
 *-----------------------------------------------------------------------------
 *
 * In these instruction, the 16 byte data field is interepreted as follows.
 *
 * This command is used to scroll all the pixels on the screen horizontally and/or
 * vertically.
 *
 * The color refers to a fill color to use for the new area uncovered by the
 * scrolling action.  It is only used in the Scroll Preset command.  In the Scroll
 * Copy command the screen is "rotated" around.  For example, in scrolling to the
 * left, pixels uncovered on the right are filled in by the pixels being scrolled
 * off the screen on the left.
 *
 * The hScroll field is a compound field. It can be divided into two fields like so:
 *
 *   SCmd = (hScroll & 0x30) >> 4;
 *   HOffset = (hScroll & 0x07);
 *
 *      SCmd is a scrolliing instruction, which is either 0, 1 or 2.
 *           0 means don't scroll
 *           1 means scroll 6 pixels to the right,
 *           2 means scroll 6 pixels to the left.
 *
 *      HOffset is a horizontal offset which is used for offsetting the graphic
 *      display by amounts less than 6 pixels. It can assume values from 0 to 5.
 *
 * Similarly, the vScroll field is a compound field. It can be divided into two
 * fields like so:
 *
 *   SCmd = (vScroll & 0x30) >> 4;
 *   VOffset = (vScroll & 0x0F);
 *
 *      SCmd is a scrolliing instruction, which is either 0, 1 or 2.
 *           0 means don't scroll
 *           1 means scroll 12 pixels down,
 *           2 means scroll 12 pixels up.
 *
 *      VOffset is a vertical offset which is used for offsetting the graphic
 *      display by amounts less than 12 pixels. It can assume values from 0 to 11.
 *
 * Smooth horizontal and vertical scrolling in all directions can be done by
 * combining scroll commands.  For example, here is a smooth horizontal scroll to
 * the left:
 *
 *      SCmd           HScroll
 *     ===             =======
 *      0                   1
 *      0                   2
 *      0                   3
 *      0                   4
 *      0                   5
 *      0                   6
 *      2                   0
 *      (repeat)
 *
 * You can create the effect of an infinite panorama by continually loading in new
 * tiles into the border area and scrolling them into view.
 *
 *****************************************************************************/

#define  SCROLL_PRESET_SUBCODE      20
#define  SCROLL_COPY_SUBCODE        24
#define  SCROLL_COLOR_MASK          0x0F
#define  SCROLL_HSCROLL_MASK        0x3F
#define  SCROLL_VSCROLL_MASK        0x3F

typedef struct {
   char  color;                              // Only lower 4 bits are used, mask with 0x0F
   char  hScroll;                            // Only lower 6 bits are used, mask with 0x3F
   char  vScroll;                            // Only lower 6 bits are used, mask with 0x3F
} CDG_Scroll;

typedef enum {
   SCROLL_PRESET,
   SCROLL_COPY
} Scroll_Actions;

#define GetSColor(a)         ((a).scroll.color   & SCROLL_COLOR_MASK)
#define GetShScroll(a)       ((a).scroll.hScroll & SCROLL_HSCROLL_MASK)
#define GetSvScroll(a)       ((a).scroll.vScroll & SCROLL_VSCROLL_MASK)

/******************************************************************************
 *
 * Define Transparent Color (subCode.instruction==28)
 *
 *    Define a specific color as being transparent.
 *
 *-----------------------------------------------------------------------------
 *
 * This command is used to define a CLUT color as being transparent, for example so
 * that the the graphics can be overlayed on top of a live video signal.
 *
 *****************************************************************************/

#define  DEFINE_TRASPARENT_COLOR_SUBCODE     28

typedef  unsigned char                       CDG_TransparentColor;

/******************************************************************************
 *
 * Load Color Table Lo   [colors 0 thru 7]  (subCode.instruction==30)
 *
 *    Load in the lower 8 entries of the color table.
 *
 * Load Color Table High [colors 8 thru 15] (subCode.instruction==31)
 *
 *    Load in the upper 8 entries of the color table.
 *
 *-----------------------------------------------------------------------------
 *
 * These commands are used to load in the colors for the color table.  The colors
 * are specified using 4 bits each for R, G and B, resulting in 4096 possible
 * colors.  The only different between the two instructions is whether the low
 * part or the high part of the color table is loaded.
 *
 * In these instruction, the 16 byte data field is interpreted as follows.
 *
 * Each colorSpec value can be converted to RGB using the following diagram:
 *
 *     [---high byte---]   [---low byte----]
 *      7 6 5 4 3 2 1 0     7 6 5 4 3 2 1 0
 *      X X r r r r g g     X X g g b b b b
 *
 * Note that P and Q channel bits need to be masked off (they are marked
 * here with Xs.
 *
 * The Load CLUT commands are often used in CD+G to provide the illusion of
 * animation through the use of color cyling.
 *
 *****************************************************************************/

#define  LOAD_COLOR_TABLE_LO_SUBCODE     30
#define  LOAD_COLOR_TABLE_HIGH_SUBCODE   31
#define  LOAD_COLOR_TABLE_COLOR_SZ        8
#define  LOAD_COLOR_TABLE_COLOR_SPEC_SZ  16
#define  LOAD_COLOR_TABLE_COLOR_MASK     0x3F3F

typedef struct {
   unsigned char color[LOAD_COLOR_TABLE_COLOR_SPEC_SZ];   // AND with 0x3F3F to clear P and Q channel
} CDG_LoadCLUT;

typedef enum {
   LOAD_COLOR_TABLE_LO,
   LOAD_COLOR_TABLE_HI
} LoadColorTable_Actions;

#define GetLSColor(a, b)     ((a).loadCLUT.color[(b)] & LOAD_COLOR_TABLE_COLOR_MASK)

/******************************************************************************
 *
 * The 96 bytes of Subchannel information in each sector contain 4 packets of 24
 * bytes apiece.
 *
 * Each byte in the 16 byte SubCode.data[] field can be thought of as being divided
 * into 8-bits.  Each of these bits corresponds to a separate stream of information.
 * These streams are called "channels", and are labeled starting with the letter P,
 * like so:
 *
 *   Channel#      P Q R S T U V W
 *   Bit#          7 6 5 4 3 2 1 0
 *                 ---------------
 *   Byte #  0     0 0 0 0 0 0 0 0
 *   Byte #  1     0 0 0 0 0 0 0 0
 *   Byte #  2     0 0 0 0 0 0 0 0
 *   .
 *   . (etc.)
 *   .
 *   Byte #  15    0 0 0 0 0 0 0 0
 *
 * Both the P and Q channels, on a regular Audio CD are used for timing information.
 * They are used to assist the CD Player in tracking the current location on the
 * disc, and to provide the timing information for the time display on the CD
 * Player.
 *
 * On a regular Audio CD, channels R through W are unused.
 *
 * The CD+G format takes advantage of the unused channels R thru W.  These unused
 * six bits are used to store graphics information. Note that this is an extremely
 * thin stream of information.  6 bits per byte * 16 bytes per packet * 4
 * packets per sector * 75 sectors per second = 28800 bits per second, or 3.6 K per
 * second.  By comparison, a typical 160 x 120 QuickTime movie uses 90K per second.
 *
 ******************************************************************************/

#define  SC_MASK                          0x3F
#define  SC_CDG_COMMAND                   0x09

#define  SUBCHANNEL_Q_PARITY_SIZE         2
#define  SUBCHANNEL_P_PARITY_SIZE         4
#define  SUBCODE_DATA_SZ                  16

typedef union {
   unsigned char        filler[SUBCODE_DATA_SZ];
   CDG_MemPreset        memPreset;
   CDG_BorderPreset     borderPreset;
   CDG_Tile             tile;
   CDG_Scroll           scroll;
   CDG_LoadCLUT         loadCLUT;
   CDG_TransparentColor transColor;
} SubCodeData;

typedef struct {
	char           command;
	char           instruction;
	char           parityQ[SUBCHANNEL_Q_PARITY_SIZE];
	SubCodeData    data;
	char           parityP[SUBCHANNEL_P_PARITY_SIZE];
} SubCode;

typedef enum {
   CDG_MEMORYPRESET           = MEMORY_PRESET_SUBCODE,
   CDG_BORDERPRESET           = BORDER_PRESET_SUBCODE,
   CDG_TILEBLOCKNORMAL        = TILE_BLOCK_NORMAL_SUBCODE,
   CDG_SCROLLPRESET           = SCROLL_PRESET_SUBCODE,
   CDG_SCROLLCOPY             = SCROLL_COPY_SUBCODE,
   CDG_DEFINETRASPARENTCOLOR  = DEFINE_TRASPARENT_COLOR_SUBCODE,
   CDG_LOADCOLORTABLELO       = LOAD_COLOR_TABLE_LO_SUBCODE,
   CDG_LOADCOLORTABLEHIGH     = LOAD_COLOR_TABLE_HIGH_SUBCODE,
   CDG_TILEBLOCKXOR           = TILE_BLOCK_XOR_SUBCODE
} SubCode_Instructions;

#define GetSCCmd(a)          ((a).command      & SC_MASK)
#define GetSCInstruction(a)  ((a).instructions & SC_MASK)
#define GetSCData(a)         ((a).data)

/*******************************************************************************
 * General declerations
 ******************************************************************************/

#define SECTORS_PER_SECOND    75
#define CONTENT_DATA_SZ       2352

#define SC_DATA_SZ            96    // The 96 bytes of Subchannel information
#define SC_PACKET_PER_SECTOR  4     // in each sector contain 4 packets
#define SC_PACKET_SZ          24    // of 24 bytes apiece.

                                    // each sector has 2352 bytes of Content data
                                    // and 96 bytes of Subchannel data.
#define SECTOR_SZ             (SC_DATA_SZ + CONTENT_DATA_SZ)

typedef struct {
   char     content[CONTENT_DATA_SZ];
   SubCode  subCode[SC_PACKET_PER_SECTOR];
} TrackFileSector;

typedef union {
	unsigned char       raw[SC_PACKET_SZ];
	SubCode				subCode;
} SubCodePacket;

typedef unsigned char boolean;

#define CDG_SUM_MEMORYPRESET           0
#define CDG_SUM_BORDERPRESET           1
#define CDG_SUM_TILEBLOCKNORMAL        2
#define CDG_SUM_SCROLLPRESET           3
#define CDG_SUM_SCROLLCOPY             4
#define CDG_SUM_DEFINETRASPARENTCOLOR  5
#define CDG_SUM_LOADCOLORTABLELO       6
#define CDG_SUM_LOADTABLEHI		       7
#define CDG_SUM_TILEBLOCKXOR           8
#define CDG_SUM_UNDEFINES              9
#define CDG_SUM_NUMBER_OF_COMMANDS     10

/*****************************************************************************
 *
 * Defines used for paackatization
 *
 ****************************************************************************/
#define CDG_MIN_COLOR_INDEX			 0
#define CDG_MAX_COLOR_INDEX			 15


#define NO_COLOR                    (unsigned int)0

#define MAX_SCREEN_ROW_TILES        50
#define MAX_SCREEN_COLUMN_TILES     18
#define MAX_VIEW_ROW_TILES          48
#define MAX_VIEW_COLUMN_TILES       16
#define MAX_VERTICAL_SCROLL         12
#define MAX_HORIZONTAL_SCROLL        6
 
#define MAX_VERTICAL_SCROLL_DOWN     0x10
#define MAX_VERTICAL_SCROLL_UP       0x20
#define MAX_HORIZONTAL_SCROLL_RIGHT  0x10
#define MAX_HORIZONTAL_SCROLL_LEFT   0x20

#define  CLEAR_MASK                 0xC0
#define  COLOR_MASK                 0x0F
#define  REPEAT_MASK                0x0F
#define  ROW_MASK                   0x1F
#define  COLUMN_MASK                0x3F
#define  PIXELS_MASK                0x3F
#define  SCROLL_MASK                0x3F
#define  BITMAP_ROW_MASK            0x3F
#define  CLUT_COLOR_MASK            0x3F
#define  LO_CLUT_MASK               (unsigned short)0x003F
#define  HIGH_CLUT_MASK             (unsigned short)0x0FC0
#define  HIGH_CLUT_OFFSET           6

#define GetColor(a)                 ((a) & COLOR_MASK)
#define GetRepeat(a)                ((a) & REPEAT_MASK)
#define GetRow(a)                   ((a) & ROW_MASK)
#define GetColumn(a)                ((a) & COLUMN_MASK)
#define GetScroll(a)                ((a) & SCROLL_MASK)
#define GetColorTableColor(a)       ((a) & COLOR_TABLE_MASK)

#define PutColor(a, color)          ((a) = (((unsigned char)color)     & COLOR_MASK)      \
                                         + ((a) & ~COLOR_MASK))
#define PutRepeat(a, repeat)        ((a) = (((unsigned char)repeat)    & REPEAT_MASK)     \
                                         + ((a) & ~REPEAT_MASK))
#define PutRow(a, row)              ((a) = (((unsigned char)row)       & ROW_MASK)        \
                                         + ((a) & ~ROW_MASK))
#define PutColumn(a, column)        ((a) = (((unsigned char)column)    & COLUMN_MASK)     \
                                         + ((a) & ~COLUMN_MASK))
#define PutScroll(a, scroll)        ((a) = (((unsigned char)scroll)    & SCROLL_MASK)     \
                                         + ((a) & ~SCROLL_MASK))
#define PutBitmapRow(a, bitMapRow)  ((a) = (((unsigned char)bitMapRow) & BITMAP_ROW_MASK) \
                                         + ((a) & ~BITMAP_ROW_MASK))
#define PutCLUTColor(a, color)      ((a) = (((unsigned char)color)     & CLUT_COLOR_MASK) \
                                         + ((a) & ~CLUT_COLOR_MASK))

#define PutCommand(a)               ((a.command)     = SC_CDG_COMMAND)
#define PutInstruction(a, inst)     ((a.instruction) = (inst))

#define ClearCDG(a)                 ((a) = ((a) & CLEAR_MASK))

#endif
