/*  
 *
 *  (c) 1999 Norm Bell
 *  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  cdgpacker.h : Defines the API for the cdgpacker library.
 *
 *  permission is granted for free use this demo software to develop 
 *  applications as long as the applications are given away
 *  for free and are not sold.
 */




/******************************************************************************
 *
 * Package a BorderPreset Command
 *
 *    Set the border of the screen to a particular color.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color	:	index into color table,
 *					valid range is 0 to 15
 *		packet	:	pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		:	successfull completion
 *		-1		:	undefined packet pointer
 *		-2		:	color table index out of range
 *****************************************************************************/

 int  PackBorderPreset    (unsigned int   color,
                                        unsigned char *packet);


/******************************************************************************
 *
 * Package Load Color Table Lo Command
 * Package Load Color Table High Command
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		colorTable	:  pointer to 16 byte array containing 8, 12 bit RGB values
 *		packet	    :  pointer to 24 byte array to receive packed command,
 *	                   update done in place
 *
 * Returns:
 *		 0		:	Successfull completion
 *		-1		:	undefined packet pointer
 *		-2		:	undefined colorTable pointer
 *****************************************************************************/

 int  PackColorTableLo    (unsigned short *colorTable,
                                        unsigned char  *packet);
 int  PackColorTableHigh  (unsigned short *colorTable,
                                        unsigned char  *packet);


/******************************************************************************
 *
 * Initialize CDG Packet
 *
 *    Initialize CDG Packet to zero.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		packet	:	pointer to 24 byte array intialized in place,
 *					update done in place
 *
 * Returns:
 *		 None
 *****************************************************************************/

 void PackInitialize      (unsigned char *packet);


/******************************************************************************
 *
 * Package a MemoryPreset Command
 *
 *    Set the screen to a particular color.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color	:	index into color table,
 *					valid range is 0 to 15
 *		packet	:	pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		:	successfull completion
 *		-1		:	undefined packet pointer
 *		-2		:	color table index out of range
 *****************************************************************************/

 int  PackMemoryPreset    (unsigned int   color,
                                        unsigned int   sequenceNumber,
                                        unsigned char *packet);


/******************************************************************************
 *
 * 
 * Package Scroll Preset Command
 *
 *    Scroll the image, filling in the new area with a color.
 *
 * Package Scroll Copy Command
 *
 *    Scroll the image, rotating the bits back around.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color	 :	index into color table for 0 bits,
 *					valid range is 0 to 15
 *      direction:	direction of scroll,
 *					valid range is one of SCROLL_UP, SCROLL_DOWN, SCROLL_LEFT,
 *					or SCROLL_RIGHT
 *      pixels   :	number of pixels to scroll,
 *					valid range for horizontal scroll is 0 to 6,
 *					valid range for vertical scroll is 0 to 12
 *		packet[] :  pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		 :  successfull completion
 *		-1		 :  undefined packet pointer
 *		-2		 :  color table index out of range
 *      -3       :  pixel count out of range
 *      -4       :  invalid scrolling direction
 *****************************************************************************/

typedef enum {
    SCROLL_UP,
    SCROLL_DOWN,
    SCROLL_LEFT,
    SCROLL_RIGHT
}
Scroll_Control;

 int PackScrollCopy       (Scroll_Control direction,
                                        unsigned int   pixels,
                                        unsigned char *packet);
 int PackScrollPreset     (Scroll_Control direction,
                                        unsigned int   color,
                                        unsigned int   pixels,
                                        unsigned char *packet);


/******************************************************************************
 *
 * Package Tile Block Normal Command
 *
 *    Load a 12 x 6, 2 color tile and display by direct copying.
 *
 * Package Tile Block XOR Command
 *
 *    Load a 12 x 6, 2 color tile and display it using the XOR method.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color0	 :	index into color table for 0 bits,
 *					valid range is 0 to 15
 *		color1	 :	index into color table for 1 bits,
 *					valid range is 0 to 15
 *      row      :  horizontal position of the tile in tile coordinate space,
 *                  valid range is 0 to 50
 *      column   :  vertical position of the tile in tile coordinate space,
 *                  valid range is 0 to 18
 *      bitmap[] :  pointer to 12 byte array containing the actual bit values
 *                  for the tile, six pixels per byte. The uppermost valid
 *                  bit of each byte (0x20) contains the left-most pixel of
 *                  each scanline of the tile
 *		packet[] :  pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		 :  successfull completion
 *		-1		 :  undefined packet pointer
 *		-2		 :  color0 table index out of range
 *		-3		 :  color1 table index out of range
 *      -4       :  undefined bitmap pointer
 *      -5       :  row value out of range
 *      -6       :  column value out of range
 *****************************************************************************/

 int  PackTileNormal      (unsigned int   color0,
                                        unsigned int   color1,
                                        unsigned int   row,
                                        unsigned int   column,
                                        unsigned char *bitmap,
                                        unsigned char *packet);
 int  PackTileXOR         (unsigned int   color0,
                                        unsigned int   color1,
                                        unsigned int   row,
                                        unsigned int   column,
                                        unsigned char *bitmap,
                                        unsigned char *packet);


/******************************************************************************
 *
 * Package Define Trasparent Color Command
 *
 *    Indicate the specified color is to be treated as transparent.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color	:	index into color table,
 *					valid range is 0 to 15
 *		packet	:	pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		:	successfull completion
 *		-1		:	undefined packet pointer
 *		-2		:	color table index out of range
 *****************************************************************************/

 int PackTransperentColor (unsigned int   color,
                                        unsigned char *packet);

